CtA_Hawk = LibStub("AceAddon-3.0"):NewAddon("CtA_Hawk", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("CtA_Hawk", true)
local LSM = LibStub("LibSharedMedia-3.0")

LSM:Register("sound", "War Drums", [[Sound\Event Sounds\Event_wardrum_ogre.wav]])

local DB_VERSION = 3

local UPDATEPERIOD, elapsed = 10, 0

CtA_Hawk.display = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("CtA Hawk", {type = "data source", label=L["CtA"], text = L["Waiting..."]})

CtA_Hawk.updateframe = CreateFrame("frame")

local dungeonGroups = {}
CtA_Hawk.dungeonGroups = dungeonGroups

local RARE = LFG_ROLE_SHORTAGE_RARE
local UNCOMMON = LFG_ROLE_SHORTAGE_UNCOMMON
local PLENTIFUL = LFG_ROLE_SHORTAGE_PLENTIFUL

local shortageColor = {
	[RARE] = "|cFFFF7F00",
	[UNCOMMON] = "|cFFFFFF00",
	[PLENTIFUL] = "|cFF7FFF00",
	Unknown = "|cFF555555",
	NoCtA = "|cFFFFFFFF",
}

local shortageName = {
	[RARE] = shortageColor[RARE]..L["Rare"],
	[UNCOMMON] = shortageColor[UNCOMMON]..L["Uncommon"],
	[PLENTIFUL] = shortageColor[PLENTIFUL]..L["Plentiful"],
	Unknown = shortageColor.Unknown..L["Unknown"],
}

local availableRoles = {}

local roleIcons = {
	Tank = "|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:0:19:21:40|t",
	Healer = "|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:20:39:0:19|t",
	DPS = "|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:20:39:21:40|t",
}
roleIcons.NoCtA = roleIcons.Tank..roleIcons.Healer..roleIcons.DPS

function CtA_Hawk:OnInitialize()

	if CtA_HawkOptions ~= nil then 
		if (CtA_HawkOptions.version or 0) ~= DB_VERSION then CtA_HawkOptions = {version = DB_VERSION} end
	end

	availableRoles["Tank"], availableRoles["Healer"], availableRoles["DPS"] = UnitGetAvailableRoles("player")
	local roleOptions = {}
	if availableRoles["Tank"] then roleOptions.Tank = L["Tank"] end
	if availableRoles["Healer"] then roleOptions.Healer = L["Healer"] end
	if availableRoles["DPS"] then roleOptions.DPS = L["DPS"] end

	self.options = {
		name = L["CtA Hawk: Options"],
		type = "group",
		args = {
			settings = {
				name = "settings",
				type = "group",
				args = {
					enableAlerts = {
						type = "toggle",
						name = L["Enable Alerts"],
						desc = L["Check this box to enable alerts. Configure the alert rules for each role below."],
						set = function(info,val)
							CtA_Hawk.db.profile.enableAlerts = val
						end,
						get = function(info) return CtA_Hawk.db.profile.enableAlerts end,
						order = 0,
					},
					quietInGroup = {
						type = "toggle",
						name = L["Quiet In Group"],
						desc = L["Check this box to suppress alerts when you are in a group."],
						disabled = function() return not CtA_Hawk.db.profile.enableAlerts end,
						set = function(info,val)
							CtA_Hawk.db.profile.quietInGroup = val
						end,
						get = function(info) return CtA_Hawk.db.profile.quietInGroup end,
						order = 0,
					},
					enableAutoQueue = {
						type = "toggle",
						name = L["Enable Auto-Queue"],
						desc = L["EnableAutoQueueNote1"].."\n|cFFFFCC00"..L["EnableAutoQueueNote2"],
						descStyle = "inline",
						disabled = function()
							if GetLFGMode(LE_LFG_CATEGORY_LFD) ~= nil or GetNumGroupMembers() > 0 then return true end
							if CtA_Hawk.db.profile.Tank.autoQueue[RARE] then return false end
							if CtA_Hawk.db.profile.Tank.autoQueue[RARE] then return false end
							if CtA_Hawk.db.profile.Tank.autoQueue[RARE] then return false end
							if CtA_Hawk.db.profile.Healer.autoQueue[UNCOMMON] then return false end
							if CtA_Hawk.db.profile.Healer.autoQueue[UNCOMMON] then return false end
							if CtA_Hawk.db.profile.Healer.autoQueue[UNCOMMON] then return false end
							if CtA_Hawk.db.profile.DPS.autoQueue[PLENTIFUL] then return false end
							if CtA_Hawk.db.profile.DPS.autoQueue[PLENTIFUL] then return false end
							if CtA_Hawk.db.profile.DPS.autoQueue[PLENTIFUL] then return false end
						 	if CtA_Hawk.db.profile.NoCtA.autoQueue.Tank then return false end
							if CtA_Hawk.db.profile.NoCtA.autoQueue.Healer then return false end
							if CtA_Hawk.db.profile.NoCtA.autoQueue.DPS then return false end
							return true
						end,
						width = "full",
						set = function(info,val)
							if val then
								CtA_Hawk:EnableAutoQueue()
							else
								CtA_Hawk:DisableAutoQueue()
							end
						end,
						get = function(info) return CtA_Hawk.autoQueueEnabled end,
						order = 1,
					},
					general = {
						type = "group",
						name = L["General"],
						order = 1,
						args = {
							ldbDisplay = {
								type = "group",
								name = L["LDB Display Options"],
								inline = true,
								order = 2,
								args = {
									label = {
										type = "select",
										name = L["LDB Label"],
										desc = L["Choose what should be shown as the LDB label."],
										values = {
											Short = L["CtA"],
											Full = L["Call To Arms"],
										},
										order = 1,
										set = function(info, val)
											CtA_Hawk.db.profile.ldbLabel = val
											CtA_Hawk:RefreshLDB()
										end,
										get = function()
											return CtA_Hawk.db.profile.ldbLabel
										end,
									},
									data = {
										type = "select",
										name = L["LDB Data"],
										desc = L["Choose how the roles are shown in the LDB text."],
										values = {
											Short = L["Short Text (T/H/D)"],
											Full = L["Full Text (Tank/Healer/DPS)"],
											Icons = string.format("%s (%s/%s/%s)", L["Icons"], roleIcons.Tank, roleIcons.Healer, roleIcons.DPS),
											FullWithIcons = L["Full with icons"],
										},
										order = 2,
										set = function(info, val)
											CtA_Hawk.db.profile.ldbText = val
											CtA_Hawk:RefreshLDB()
										end,
										get = function()
											return CtA_Hawk.db.profile.ldbText
										end,
									},
								},
							},
						},
					},
					queues = {
						type = "group",
						name = L["Dungeon Groups"],
						desc = L["Select the dungeon queues for which you want to monitor role status."],
						order = 2,
						args = {},
					},
					NoCtA = {
						type = "group",
						name = L["NoCtA"],
						order = 3,
						args = {
							autoQueue = {
								type = "multiselect",
								name = L["Auto-Queue As"],
								desc = L["Automatically enter the queue for this role when there is no Call to Arms."],
								order = 1,
								width = "full",
								values = roleOptions,
								set = function (_, key, val)
									CtA_Hawk.db.profile.NoCtA.autoQueue[key] = val
									if val then CtA_Hawk:ScanForAutoQueue() end
								end,
								get = function (_, key)
									return CtA_Hawk.db.profile.NoCtA.autoQueue[key]
								end,
							},
							text = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Text Alert"],
								desc = L["Display message on screen when there is no Call to Arms."],
								descStyle = "inline",
								order = 4,
								set = function(info, val)
									CtA_Hawk.db.profile.NoCtA.textNotify = val
									if val and not CtA_Hawk.db.profile.NoCtA.chatNotify and CtA_Hawk.db.profile.NoCtA.soundFile == "None" then CtA_Hawk:RoleMonitoringEnabled("NoCtA") end
									CtA_Hawk:RefreshLDB()
								end,
								get = function()
									return CtA_Hawk.db.profile.NoCtA.textNotify
								end,
							},
							chat = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Chat Alert"],
								desc = L["Display message in chat window when there is no Call to Arms."],
								descStyle = "inline",
								order = 5,
								set = function(info, val)
									CtA_Hawk.db.profile.NoCtA.chatNotify = val
									if val and not CtA_Hawk.db.profile.NoCtA.textNotify and CtA_Hawk.db.profile.NoCtA.soundFile == "None" then CtA_Hawk:RoleMonitoringEnabled("NoCtA") end
									CtA_Hawk:RefreshLDB()
								end,
								get = function()
									return CtA_Hawk.db.profile.NoCtA.chatNotify
								end,
							},
							soundFile = {
								type = "select",
								dialogControl = "LSM30_Sound",
								name = L["Audible Alert"],
								desc = L["Select a sound file to be played when there is no Call to Arms."],
								values = AceGUIWidgetLSMlists.sound,
								get = function()
									return current
								end,
								set = function(self,key)
									current = key
									if val and not CtA_Hawk.db.profile.NoCtA.chatNotify and not CtA_Hawk.db.profile.NoCtA.textNotify then CtA_Hawk:RoleMonitoringEnabled("NoCtA") end
								end,
								order = 7,
								set = function(info, val)
									CtA_Hawk.db.profile.NoCtA.soundFile = val
								end,
								get = function()
									return CtA_Hawk.db.profile.NoCtA.soundFile
								end,
							},
						},
					},
					Tank = {
						type = "group",
						name = roleIcons.Tank..L["Tank"],
						order = 4,
						args = {
							autoQueue = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Auto-Queue When'],
								desc = L["AutoQueueNote1"].."\n|cFFFFCC00"..L["AutoQueueNote2"],
								disabled = not availableRoles.Tank,
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 1,
								set = function(info, key, val)
									CtA_Hawk.db.profile.Tank.autoQueue[key] = val
									if val then CtA_Hawk:ScanForAutoQueue() end
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.Tank.autoQueue[key]
								end,
							},
							notify = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Notify When'],
								desc = L["Alerts you when this role enters this status. The options below control the type of notification."],
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 2,
								set = function(info, key, val)
									CtA_Hawk.db.profile.Tank.notify[key] = val
									if val then CtA_Hawk:RoleMonitoringEnabled(key, "Tank") end
									CtA_Hawk:RefreshLDB()
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.Tank.notify[key]
								end,
							},
							text = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Text Alert"],
								desc = L["Display message on screen when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 4,
								set = function(info, val)
									CtA_Hawk.db.profile.Tank.textNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Tank.textNotify
								end,
							},
							chat = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Chat Alert"],
								desc = L["Display message in chat window when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 5,
								set = function(info, val)
									CtA_Hawk.db.profile.Tank.chatNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Tank.chatNotify
								end,
							},
							soundFile = {
								type = "select",
								dialogControl = "LSM30_Sound",
								name = L["Audible Alert"],
								desc = L["Select a sound file to be played when this role enters one of the statuses selected above."],
								values = LSM:HashTable("sound"),
								get = function()
									return current
								end,
								set = function(self,key)
									current = key
								end,
								order = 7,
								set = function(info, val)
									CtA_Hawk.db.profile.Tank.soundFile = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Tank.soundFile
								end,
							},
						},
					},
					Healer = {
						type = "group",
						name = roleIcons.Healer..L["Healer"],
						order = 5,
						args = {
							autoQueue = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Auto-Queue When'],
								desc = L["AutoQueueNote1"].."\n|cFFFFCC00"..L["AutoQueueNote2"],
								disabled = not availableRoles.Healer,
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 1,
								set = function(info, key, val)
									CtA_Hawk.db.profile.Healer.autoQueue[key] = val
									if val then CtA_Hawk:ScanForAutoQueue() end
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.Healer.autoQueue[key]
								end,
							},
							notify = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Notify When'],
								desc = L["Alerts you when this role enters this status. The options below control the type of notification."],
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 2,
								set = function(info, key, val)
									CtA_Hawk.db.profile.Healer.notify[key] = val
									if val then CtA_Hawk:RoleMonitoringEnabled(key, "Healer") end
									CtA_Hawk:RefreshLDB()
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.Healer.notify[key]
								end,
							},
							text = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Text Alert"],
								desc = L["Display message on screen when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 4,
								set = function(info, val)
									CtA_Hawk.db.profile.Healer.textNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Healer.textNotify
								end,
							},
							chat = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Chat Alert"],
								desc = L["Display message in chat window when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 5,
								set = function(info, val)
									CtA_Hawk.db.profile.Healer.chatNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Healer.chatNotify
								end,
							},
							soundFile = {
								type = "select",
								dialogControl = "LSM30_Sound",
								name = L["Audible Alert"],
								desc = L["Select a sound file to be played when this role enters one of the statuses selected above."],
								values = LSM:HashTable("sound"),
								get = function()
									return current
								end,
								set = function(self,key)
									current = key
								end,
								order = 7,
								set = function(info, val)
									CtA_Hawk.db.profile.Healer.soundFile = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Healer.soundFile
								end,
							},
						},
					},
					DPS = {
						type = "group",
						name = roleIcons.DPS..L["DPS"],
						order = 6,
						args = {
							autoQueue = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Auto-Queue When'],
								desc = L["AutoQueueNote1"].."\n|cFFFFCC00"..L["AutoQueueNote2"],
								disabled = not availableRoles.DPS,
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 1,
								set = function(info, key, val)
									CtA_Hawk.db.profile.DPS.autoQueue[key] = val
									if val then CtA_Hawk:ScanForAutoQueue() end
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.DPS.autoQueue[key]
								end,
							},
							notify = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Notify When'],
								desc = L["Alerts you when this role enters this status. The options below control the type of notification."],
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 2,
								set = function(info, key, val)
									CtA_Hawk.db.profile.DPS.notify[key] = val
									if val then CtA_Hawk:RoleMonitoringEnabled(key, "DPS") end
									CtA_Hawk:RefreshLDB()
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.DPS.notify[key]
								end,
							},
							text = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Text Alert"],
								desc = L["Display message on screen when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 4,
								set = function(info, val)
									CtA_Hawk.db.profile.DPS.textNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.DPS.textNotify
								end,
							},
							chat = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Chat Alert"],
								desc = L["Display message in chat window when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 5,
								set = function(info, val)
									CtA_Hawk.db.profile.DPS.chatNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.DPS.chatNotify
								end,
							},
							soundFile = {
								type = "select",
								dialogControl = "LSM30_Sound",
								name = L["Audible Alert"],
								desc = L["Select a sound file to be played when this role enters one of the statuses selected above."],
								values = AceGUIWidgetLSMlists.sound,
								get = function()
									return current
								end,
								set = function(self,key)
									current = key
								end,
								order = 7,
								set = function(info, val)
									CtA_Hawk.db.profile.DPS.soundFile = val
								end,
								get = function()
									return CtA_Hawk.db.profile.DPS.soundFile
								end,
							},
						},
					},
				},
			},
		},
	}
	
	self.defaults = {
		profile = {
			enableAlerts = true,
				quietInGroup = true,
			ldbLabel = "Full",
			ldbText = "Full",
			queues = {
				["auto"] = true,
			},
			Tank = {
				autoQueue = {},
				notify = availableRoles.Tank and {[RARE] = true} or {},
				textNotify = true,
				chatNotify = true,
				soundFile = "War Drums",
			},
			Healer = {
				autoQueue = {},
				notify = availableRoles.Healer and {[RARE] = true} or {},
				textNotify = true,
				chatNotify = true,
				soundFile = "War Drums",
			},
			DPS = {
				autoQueue = {},
				notify = availableRoles.DPS and {[RARE] = true} or {},
				textNotify = true,
				chatNotify = true,
 				soundFile = "War Drums",
			},
			NoCtA = {
				autoQueue = {},
				textNotify = false,
				chatNotify = false,
 				soundFile = "None",
			}
		}
	}
	
	self.db = LibStub("AceDB-3.0"):New("CtA_HawkOptions",self.defaults)
	
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("CtA_Hawk", self.options)
	
	self.settingsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CtA_Hawk", L["CtA Hawk"], nil, "settings")  
	self.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CtA_Hawk", L["Profiles"], L["CtA Hawk"], "profiles")
	
	self.updateframe:SetScript("OnUpdate",CtA_Hawk.UpdateFrameUpdate)

	self:RegisterEvent("LFG_UPDATE_RANDOM_INFO", "RefreshDungeonGroupData")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED","DisableAutoQueue")
	self:RegisterEvent("LFG_UPDATE","DisableAutoQueue")
end

function CtA_Hawk:OnLFGInitialize()

	local t = {
		["auto"] = {
			type = "toggle",
			name = "|cFFFFCC00"..L["Auto-recommend"],
			set = function(info, val)
				CtA_Hawk.db.profile.queues = {
					["auto"] = val,
					[GetRandomDungeonBestChoice()] = true,
				}
			end,
			get = function()
				return CtA_Hawk.db.profile.queues.auto
			end,
			order = 0,
		}
	}

	local numDungeons = GetNumRandomDungeons()

	for i=numDungeons,1,-1 do
		local groupID, groupName = GetLFGRandomDungeonInfo(i)
		local eligible = GetLFGRoleShortageRewards(groupID,1)
		local index = numDungeons - i + 1
		table.insert(dungeonGroups, index, {
			id = groupID,
			name = groupName,
			eligible = eligible,
		})
		t["Dungeon"..groupID] = {
			type = "toggle",
			name = strsub(groupName, 8),
			disabled = function() return CtA_Hawk.db.profile.queues.auto end,
			hidden = function() return not dungeonGroups[index].eligible end,
			set = function(info, val)
				CtA_Hawk.db.profile.queues[groupID] = val
				if val then
					CtA_Hawk:DungeonMonitoringEnabled(groupID)
					self:ScanForAutoQueue()
					CtA_Hawk:RefreshLDB()
				end
			end,
			get = function()
				return CtA_Hawk:IsDungeonGroupMonitored(groupID)
			end,
			order = 10000 - groupID,
		}
	end
	self.options.args.settings.args.queues.args = t

	self.LFGInitialized = true

	CtA_Hawk:RefreshLDB()

end

function CtA_Hawk:DisableAutoQueue()
	if self.autoQueueEnabled then
		self.autoQueueEnabled = false
		print("|cFFFFCC00"..L["Call To Arms: "].."|r"..L["Auto-Queue Disabled!"])
	end
	LibStub("AceConfigRegistry-3.0"):NotifyChange("CtA_Hawk");
end

function CtA_Hawk:EnableAutoQueue()
	if not self.autoQueueEnabled then
		self.autoQueueEnabled = true
		print("|cFFFFCC00"..L["Call To Arms: "].."|r"..L["Auto-Queue Enabled!"])
		self:ScanForAutoQueue()
	end
	LibStub("AceConfigRegistry-3.0"):NotifyChange("CtA_Hawk");
end

function CtA_Hawk.UpdateFrameUpdate(self, elap)

	if not CtA_Hawk.LFGInitialized then
		elapsed = elapsed + elap
		if elapsed < .5 then return end
		elapsed = 0
		if GetNumRandomDungeons() > 0 then 
			CtA_Hawk:OnLFGInitialize()
			RequestLFDPlayerLockInfo()
		end
		return
	end
	
	elapsed = elapsed + elap
	if elapsed < .5 then return end
	if elapsed < UPDATEPERIOD and CtA_Hawk.FirstDungeonDataReceived then return end
	elapsed = 0

	RequestLFDPlayerLockInfo()

end

function CtA_Hawk:IsDungeonGroupMonitored(dungeonID)
	if CtA_Hawk.db.profile.queues.auto then
		return dungeonID == GetRandomDungeonBestChoice()
	else
		return CtA_Hawk.db.profile.queues[dungeonID]
	end
end

function CtA_Hawk:DungeonMonitoringEnabled(groupID)

	if not self.LFGInitialized then return end

	for index, groupData in ipairs(dungeonGroups) do
		if groupID == groupData.id then
			self:ProcessDungeonStatusChange(groupData, groupData.Tank, "Tank")
			self:ProcessDungeonStatusChange(groupData, groupData.Healer, "Healer")
			self:ProcessDungeonStatusChange(groupData, groupData.DPS, "DPS")
			self:ProcessDungeonStatusChange(groupData, "NoCtA")
			break
		end
	end

end

function CtA_Hawk:RoleMonitoringEnabled(status, role)

	if not self.LFGInitialized then return end

	for index, groupData in ipairs(dungeonGroups) do
		if CtA_Hawk:IsDungeonGroupMonitored(groupData.id) then
			if groupData[role] == status or (status == "NoCtA" and groupData.noCtA) then
				self:ProcessDungeonStatusChange(groupData, status, role)
			end
		end
	end

end

function CtA_Hawk:RefreshDungeonGroupData ()

	if not self.LFGInitialized then return end

	for index, groupData in ipairs(dungeonGroups) do

		local dungeonGroup = {
			id = groupData.id,
			name = groupData.name,
			Tank = PLENTIFUL,
			Healer = PLENTIFUL,
			DPS = PLENTIFUL,
		}

		for shortageID=1,2 do
			
			local eligible, forTank, forHealer, forDPS, itemCount, money, xp = GetLFGRoleShortageRewards(groupData.id, shortageID);
			
			if eligible then dungeonGroup.eligible = true end

			if forTank then dungeonGroup.Tank = shortageID end
			if forHealer then dungeonGroup.Healer = shortageID end
			if forDPS then dungeonGroup.DPS = shortageID end

			if itemCount > 0 or money > 0 or xp > 0 then dungeonGroup.reward = true end

		end

		dungeonGroup.noCtA = dungeonGroup.Tank == PLENTIFUL and dungeonGroup.Healer == PLENTIFUL and dungeonGroup.DPS == PLENTIFUL
		
		if CtA_Hawk:IsDungeonGroupMonitored(groupData.id) then

			if dungeonGroups[index].Tank ~= dungeonGroup.Tank then self:ProcessDungeonStatusChange(dungeonGroup, dungeonGroup.Tank, "Tank") end
			if dungeonGroups[index].Healer ~= dungeonGroup.Healer then self:ProcessDungeonStatusChange(dungeonGroup, dungeonGroup.Healer, "Healer") end
			if dungeonGroups[index].DPS ~= dungeonGroup.DPS then self:ProcessDungeonStatusChange(dungeonGroup, dungeonGroup.DPS, "DPS") end

			if dungeonGroup.noCtA and not dungeonGroups[index].noCtA then
				self:ProcessDungeonStatusChange(dungeonGroup, "NoCtA")
			end

		end

		dungeonGroups[index] = dungeonGroup

	end

	if not self.FirstDungeonDataReceived then
		self.FirstDungeonDataReceived = true
	end

	self:RefreshLDB()

end

function CtA_Hawk:ProcessDungeonStatusChange(groupData, newStatus, role)

	if type(groupData) ~= "table" then
		groupData = dungeonGroups[groupData]
	end

	if not CtA_Hawk.db.profile.quietInGroup or GetNumGroupMembers() == 0 then
		local notifyOptions, message

		if newStatus == "NoCtA" then
			message = string.format("|cffffcc00%s%s%s%s|r - |cffffcc00%s", L["Call To Arms: "], roleIcons.NoCtA, shortageColor.NoCtA, L["NoCtA"], groupData.name)
			notifyOptions = self.db.profile.NoCtA
		else
			if not self.db.profile[role].notify[newStatus] then return end
			message = string.format("|cffffcc00%s%s%s%s|r - |cffffcc00%s", L["Call To Arms: "], roleIcons[role], shortageColor[newStatus], L[role], groupData.name)
			notifyOptions = self.db.profile[role]
		end

		if notifyOptions.textNotify then
			RaidNotice_AddMessage( RaidBossEmoteFrame, message, ChatTypeInfo["RAID_BOSS_EMOTE"] )
		end

		if notifyOptions.chatNotify then
			print(message)
		end

		if notifyOptions.soundFile ~= "" and notifyOptions.soundFile ~= "None" then
			PlaySoundFile(LSM:Fetch("sound",notifyOptions.soundFile),"Master")
		end
	end

end

function CtA_Hawk:RefreshLDB()
	self.display.label = self.db.profile.ldbLabel == "Short" and L["CtA"] or L["Call To Arms"]
	self.display.icon = "Interface\\ICONS\\Achievement_Arena_2v2_3"

	if not self.LFGInitialized or not self.FirstDungeonDataReceived then return end

	local ldbText

	if self.db.profile.ldbText == "Icons" then
		ldbText = {
			Tank = roleIcons.Tank,
			Healer = roleIcons.Healer,
			DPS = roleIcons.DPS,
			Unknown = "",
			NoCtA = "",
		}
	elseif self.db.profile.ldbText == "Short" then
		ldbText = {
			Tank = L["TankShort"],
			Healer = L["HealerShort"],
			DPS = L["DPSShort"],
			Unknown = L["UnknownShort"],
			NoCtA = L["NoCtAShort"],
		}
	elseif self.db.profile.ldbText == "FullWithIcons" then
		ldbText = {
			Tank = roleIcons.Tank..L["Tank"],
			Healer = roleIcons.Healer..L["Healer"],
			DPS = roleIcons.DPS..L["DPS"],
			Unknown = L["Unknown"],
			NoCtA = L["NoCtA"],
		}
	else
		ldbText = {
			Tank = L["Tank"],
			Healer = L["Healer"],
			DPS = L["DPS"],
			Unknown = L["Unknown"],
			NoCtA = L["NoCtA"],
		}
	end


	local text = ""
	if getn(dungeonGroups) == 0 then
		text = string.format("%s%s", shortageColor.Unknown, ldbText.Unknown)
	else
		local tankStatus, healerStatus, dpsStatus = PLENTIFUL, PLENTIFUL, PLENTIFUL

		for _, groupData in ipairs(dungeonGroups) do
			if self:IsDungeonGroupMonitored(groupData.id) then
				if tankStatus > groupData.Tank then tankStatus = groupData.Tank end
				if healerStatus > groupData.Healer then healerStatus = groupData.Healer end
				if dpsStatus > groupData.DPS then dpsStatus = groupData.DPS end
			end
		end

		if tankStatus == PLENTIFUL and healerStatus == PLENTIFUL and dpsStatus == PLENTIFUL then
			if text == "" then text = string.format("%s%s", shortageColor.NoCtA, ldbText.NoCtA) end
		else
			if self.db.profile.Tank.notify[tankStatus] then text = string.format("%s%s", shortageColor[tankStatus], ldbText.Tank) end
			if self.db.profile.Healer.notify[healerStatus] then text = string.format("%s %s%s", text, shortageColor[healerStatus], ldbText.Healer) end
			if self.db.profile.DPS.notify[dpsStatus] then text = string.format("%s %s%s", text, shortageColor[dpsStatus], ldbText.DPS) end
		end

	end
	self.display.text = text
end

function CtA_Hawk:ScanForAutoQueue()

	if not self.LFGInitialized then return end
	if not self.autoQueueEnabled then return end
	if GetLFGMode(LE_LFG_CATEGORY_LFD) ~= nil or GetNumGroupMembers() > 0 then return end

	local profile = self.db.profile
	local roles

	for _, groupData in ipairs(dungeonGroups) do
		if self:IsDungeonGroupMonitored(groupData.id) then
			if groupData.noCtA then
				roles = profile.autoQueue
			else
				roles = {
					Tank = profile.Tank.autoQueue[groupData.Tank],
					Healer = profile.Healer.autoQueue[groupData.Healer],
					DPS = profile.DPS.autoQueue[groupData.DPS],
				}
			end
			if not roles then return end
			if roles.Tank or roles.Healer or roles.DPS then
				self:EnterQueue(groupData.id, roles)
				break
			end
		end
	end

end

function CtA_Hawk:EnterQueue(groupID, roles)

	if roles then
		roles.Leader = GetLFGRoles()
	else
		roles.Leader, roles.Tank, roles.Healer, roles.DPS = GetLFGRoles()
	end

	SetLFGRoles(roles.Leader, roles.Tank, roles.Healer, roles.DPS)

	if groupID then
		LFDQueueFrame.type = groupID
	elseif not LFDQueueFrame.type then
		LFDQueueFrame.type = GetRandomDungeonBestChoice()
	end

	UIDropDownMenu_SetSelectedValue(LFDQueueFrameTypeDropDown, LFDQueueFrame.type)

	SetLFGDungeon(LFDQueueFrame.type)

	LFDQueueFrameRandom_UpdateFrame();

	JoinLFG()

end

function CtA_Hawk.display:OnEnter()

	local tooltip = LibStub("LibQTip-1.0"):Acquire("CtA_HawkTooltip", 6, "LEFT", "CENTER", "CENTER", "CENTER", "CENTER", "CENTER")
	local checkmark = "|TInterface\\BUTTONS\\UI-CheckBox-Check:20:20|t"
	local goldicon = "|TInterface\\MONEYFRAME\\UI-GoldIcon:16:16|t"
	self.tooltip = tooltip
	tooltip:AddHeader(L["Dungeon Group"], L["MonitoredHeader"],L["RewardHeader"], L["Tank"], L["Healer"], L["DPS"])
	for _, groupData in ipairs(dungeonGroups) do
		if groupData.eligible then
			if groupData.noCtA then
				tooltip:AddLine(
					"|cFFFFCC00"..groupData.name,
					(CtA_Hawk:IsDungeonGroupMonitored(groupData.id) and checkmark or ""),
					(groupData.reward and goldicon or ""),
					"-",
					"-",
					"-"
				)
			else
				tooltip:AddLine(
					"|cFFFFCC00"..groupData.name,
					(CtA_Hawk:IsDungeonGroupMonitored(groupData.id) and checkmark or ""),
					(groupData.reward and goldicon or ""),
					shortageName[groupData.Tank],
					shortageName[groupData.Healer],
					shortageName[groupData.DPS]
				)
			end
		end
	end
	tooltip:SmartAnchorTo(self)
	tooltip:Show()
end

function CtA_Hawk.display:OnLeave()
   -- Release the tooltip
   LibStub("LibQTip-1.0"):Release(self.tooltip)
   self.tooltip = nil
end

function CtA_Hawk.display:OnClick(button)
	if IsMouseButtonDown(button) then return end
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(CtA_Hawk.settingsFrame)
	end
	if button == "LeftButton" then
		if IsControlKeyDown() then
			if IsShiftKeyDown() then
				CtA_Hawk:EnterQueue()
			elseif CtA_Hawk.autoQueueEnabled then
				CtA_Hawk:DisableAutoQueue()
			else
				CtA_Hawk:EnableAutoQueue()
			end
		else
			ToggleLFDParentFrame()
		end
	end
end

function CtA_Hawk:Dump(...)
	if not IsAddOnLoaded("Blizzard_DebugTools") then LoadAddOn("Blizzard_DebugTools") end
	DevTools_Dump(...)
end